# coffeelint: disable=max_line_length

fs = require 'fs'
jsonfile = require 'jsonfile'
_ = require "lodash"
child_process = require 'child_process'
debug = require("debug")("omg-configloader")

defaultConfigurations =
  zmq:
    path: "/data/zmq_config.json"
    config:
      standardTransport: "tcp"
      standardAddress: "0.0.0.0"
      porthandler:
        rep: 9001
      pubsubbroker:
        pub: 9002
        sub: 9003
      reqrepbroker:
        rep: 9004
        req: 9005
      pushpullbroker:
        push: 9006
        pull: 9007

  producers:
    path: "/data/producer_config.json"
    config:
      standardTransport: "tcp"
      standardAddress: "127.0.0.1"
      push: 9007
      pull: 9008
      pubSub: 9009
      reqRep: 9010
      # ipcSocket: "ipc:///tmp/ipc_sockets/"

  server:
    path: "/data/webui_config.json"
    config:
      port: 80
      https: true
      allowHttp: false
      hasOldWebUi: false
      frontendDir: "webApp/bin"


configs = {}

writeConfig = (configPath, config) ->
  fs.writeFileSync configPath, JSON.stringify(config, undefined, 4)
  debug "wrote config #{configPath}"

# set new server config to old config if it exists
oldConfigPath = "/data/config.json"
if fs.existsSync oldConfigPath
  debug "checking old configuration file"
  oldConfig = jsonfile.readFileSync oldConfigPath
  if oldConfig.server
    debug "old config still has configuration for web server"
    if oldConfig.server.port
      debug "replacing port config for web server"
      defaultConfigurations.server.config.port = oldConfig.server.port
    if oldConfig.server.https
      debug "replacing https config for web server"
      defaultConfigurations.server.config.https = oldConfig.server.https
    if oldConfig.server.logLevel
      debug "replacing logLevel config for web server"
      defaultConfigurations.server.config.logLevel = oldConfig.server.logLevel
    # overwrite old config
    delete oldConfig.server
    debug "overwriting old config"
    writeConfig oldConfigPath, oldConfig
    # write new config
    debug "writing new server config"
    writeConfig defaultConfigurations.server.path, defaultConfigurations.server.config
else
  debug "no old config file"

_.each defaultConfigurations, (configObject, name) ->
  debug "loading config for #{name}"

  configPath = configObject.path
  defaultConfig = configObject.config

  #check if there is a config.json and create one if not
  unless fs.existsSync(configPath)
    debug "no config file for #{name} found"
    writeConfig(configPath, defaultConfig)

  configJson = fs.readFileSync(configPath).toString()
  configJson = JSON.parse(configJson)

  #update config.json if necessary
  configOutOfDate = false
  _.each defaultConfig, (subConfigObject, configObjectName) ->
    unless configJson.hasOwnProperty configObjectName
      debug "#{configObjectName} missing"
      configOutOfDate = true
      configJson[configObjectName] = subConfigObject

    subConfigJson = configJson[configObjectName]

    if _.isObject(subConfigObject)
      _.each subConfigObject, (value, propName) ->
        unless subConfigJson.hasOwnProperty propName
          console.log "#{propName} old"
          configOutOfDate = true
          subConfigJson[propName] = value

  if configOutOfDate
    debug "config out of date, writing new config..."
    writeConfig configPath, configJson
    debug "...config updated"
  else
    debug "config ok"

  configs[name] = configJson


configs.write = (name, data) ->
  if configs[name] and defaultConfigurations[name]
    configs[name] = data
    configPath = defaultConfigurations[name].path
    debug "updating #{name} with path", configPath
    writeConfig configPath, data

configs.keys = ->
  try
    serverPublicKey = fs.readFileSync "/data/curvekeys/serverPublic"
    serverPrivateKey = fs.readFileSync "/data/curvekeys/serverPrivate"
    producerPublicKey = fs.readFileSync "/data/curvekeys/producerPublic"
    producerPrivateKey = fs.readFileSync "/data/curvekeys/producerPrivate"
    clientPublicKey = fs.readFileSync "/data/curvekeys/clientPublic"
    clientPrivateKey = fs.readFileSync "/data/curvekeys/clientPrivate"
  catch e
    child_process.execSync "omgcurvekeygen"
    serverPublicKey = fs.readFileSync("/data/curvekeys/serverPublic")
    serverPrivateKey = fs.readFileSync("/data/curvekeys/serverPrivate")
    producerPublicKey = fs.readFileSync "/data/curvekeys/producerPublic"
    producerPrivateKey = fs.readFileSync "/data/curvekeys/producerPrivate"
    clientPublicKey = fs.readFileSync "/data/curvekeys/clientPublic"
    clientPrivateKey = fs.readFileSync "/data/curvekeys/clientPrivate"

  return {
    serverPublicKey: serverPublicKey?.toString()
    serverPrivateKey: serverPrivateKey?.toString()
    producerPublicKey: producerPublicKey?.toString()
    producerPrivateKey: producerPrivateKey?.toString()
    clientPublicKey: clientPublicKey?.toString()
    clientPrivateKey: clientPrivateKey?.toString()
  }

module.exports = configs
